package com.four.patient.controller;


import com.alibaba.fastjson.JSONObject;
import com.four.common.core.domain.Result;
import com.four.common.duck.communitypatients.SymptomsDrugs;
import com.four.common.duck.domain.GiftRecord;
import com.four.common.duck.gift.Gift;
import com.four.common.duck.interrogation.FollowDoctor;
import com.four.common.duck.interrogation.HistoricalConsultation;
import com.four.common.duck.interrogation.PatientEvaluator;
import com.four.common.duck.my.User;
import com.four.common.duck.response.ResponseRegistrationInformation;
import com.four.patient.service.RegistrationInformationService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@RestController
@RequestMapping("/registration")
@Log4j2
public class RegistrationInformationController {

    @Autowired
    private RegistrationInformationService registrationInformationService;

    @Autowired
    private HttpServletRequest request;

    /**
     * 查询全部科室
     * @return
     */
    @GetMapping("/ShowSymptomsDrug")
    public Result<List<SymptomsDrugs>> ShowSymptomsDrug(){
        log.info("功能名称:查询全部科室,请求URI:{},请求方法:{}",request.getRequestURI(),
                request.getMethod());
        List<SymptomsDrugs> list=registrationInformationService.ShowSymptomsDrug();
        log.info("功能名称:查询全部科室,请求URI:{},请求方法:{},返回结果:{}",request.getRequestURI(),
                request.getMethod(),JSONObject.toJSONString(list));
        return Result.success(list);
    }

    /**
     * 查询不同科室价格倒序的医生信息
     * @param medicalDepartmentId
     * @return
     */
    @PostMapping("/ShowRegistrationInformationAllPrice")
    public Result<List<ResponseRegistrationInformation>> ShowRegistrationInformationAllPrice(Long medicalDepartmentId) {
        log.info("功能名称:查询不同科室价格倒序的医生信息,请求URI:{},请求方法:{},请求参数:{}", request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(medicalDepartmentId));
        List<ResponseRegistrationInformation> list = registrationInformationService.ShowRegistrationInformationAllPrice(medicalDepartmentId);
        log.info("功能名称:查询不同科室价格倒序的医生信息,请求URI:{},请求方法:{},请求参数:{}", request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(list));
        return Result.success(list);
    }


    /**
     * 查询不同科室接待患者人数倒序的医生信息
     * @param medicalDepartmentId
     * @return
     */
    @PostMapping("/ShowRegistrationInformationNumber")
    public Result<List<ResponseRegistrationInformation>> ShowRegistrationInformationNumber(Long medicalDepartmentId) {
        log.info("功能名称:查询不同科室接待患者人数倒序的医生信息,请求URI:{},请求方法:{},请求参数:{}", request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(medicalDepartmentId));
        List<ResponseRegistrationInformation> list = registrationInformationService.ShowRegistrationInformationNumber(medicalDepartmentId);
        log.info("功能名称:查询不同科室接待患者人数倒序的医生信息,请求URI:{},请求方法:{},请求参数:{}", request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(list));
        return Result.success(list);
    }

    /**
     * 查询不同科室好评人数倒序排序的医生信息
     * @param medicalDepartmentId
     * @return
     */
    @PostMapping("/ShowRegistrationInformationPraise")
    public Result<List<ResponseRegistrationInformation>> ShowRegistrationInformationPraise(Long medicalDepartmentId) {
        log.info("功能名称:查询不同科室好评人数倒序排序的医生信息,请求URI:{},请求方法:{},请求参数:{}", request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(medicalDepartmentId));
        List<ResponseRegistrationInformation> list = registrationInformationService.ShowRegistrationInformationPraise(medicalDepartmentId);
        log.info("功能名称:查询不同科室好评人数倒序排序的医生信息,请求URI:{},请求方法:{},返回结果:{}", request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(medicalDepartmentId));
        return Result.success(list);
    }

    @PostMapping("/ShowRegistrationInformationAll")
    public Result<List<ResponseRegistrationInformation>> ShowRegistrationInformationAll(Long medicalDepartmentId){
        log.info("功能名称:查询不同科室医生综合排序,请求URI:{},请求方法:{}",request.getRequestURI(),
                request.getMethod());
        List<ResponseRegistrationInformation> list=registrationInformationService.ShowRegistrationInformationAll(medicalDepartmentId);
        log.info("功能名称:查询不同科室医生综合排序,请求URI:{},请求方法:{},返回结果:{}",request.getRequestURI(),
                request.getMethod(),JSONObject.toJSONString(list));
        return Result.success(list);
    }

    /**
     * 查看医生详情
     * @param registrationInformationId
     * @return
     */
    @PostMapping("/ShowRegistration")
    public Result<List<ResponseRegistrationInformation>> ShowRegistration(Long registrationInformationId){
        log.info("功能名称:查看医生详情,请求URI:{},请求方法:{},请求参数:{}",request.getRequestURI(),
                request.getMethod(),JSONObject.toJSONString(registrationInformationId));
        List<ResponseRegistrationInformation> list=registrationInformationService.ShowRegistration(registrationInformationId);
        log.info("功能名称:查看医生详情,请求URI:{},请求方法:{},返回结果:{}",request.getRequestURI(),
                request.getMethod(),JSONObject.toJSONString(list));
        return Result.success(list);
    }


}
